/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2013 digitalSTROM.org, Zurich, Switzerland
 *  Author: Sibylle Kohler
 */

var RemoteConnectivity;
var autoRefesh;
var verificationDelay;

Ext.define('DSS.addon.RemoteConnectivity', {
	extend : 'DSS.addon.Framework',

	config : {
		appName : _('my.digitalSTROM'),
		appId : 'system-addon-remote-connectivity',
		appVersion : '1.1.5',
		appIcon : 'images/app_icon.png'
	},
       
	accountAvailable : false,
	userAccount : '',
	userPwd : '',
	autoBackupActivated : false,
	remoteConnectivityEnabled : false,
	LoginDialog : undefined,
	tempLinkValidity : '',

    /**
     * The help content generation function.
     * 
     * @return {Object} The instantiated help content object.
     */
    getHelp : function() {
        return Ext.create('Ext.Component', {
            whiteSpace : 'normal',
            padding : '0 20 0 0', //top right bottom left
            loader : {
                url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad : true,
                disableCaching : false
            }
        });
    },

    /**
     * Creates the panel for step 1: 'create user account' of login panel.
     * 
     * @return {Object} The instantiated panel object.
     */
    makeStep1Panel : function() {
        var step1Description = Ext.create('Ext.form.Panel', {
            border : 0,
            html : '<span style="color:#005138;">' 
                + sStep1Titel +'<br>' + sStep1Text
                + '</span>',
            padding : '0 0 5 0', //top right bottom left
            style : {
                color : '#005138'
            },
            width : textFieldWitdh
        });
        var accountFoundHint = Ext.create('Ext.form.field.Display', {
            border : 0,
            fieldLabel : _(''),
            width : textFieldWitdh,
            labelWidth : 0,
            hidden : true,
            value : sAccountFoundText,
            fieldStyle: {fontWeight : 'bold'}
        });

        var accountNotFoundHint = Ext.create('Ext.form.field.Display', {
            border : 0,
            fieldLabel : _(''),
            width : textFieldWitdh,
            labelWidth : 0,
            hidden : true,
            value : sNoAccountFoundText,
            fieldStyle: {fontWeight : 'bold'}
        });

        var linkTermsAndConditions = Ext.create('Ext.form.Panel', {
            border : 0,
            html : '<a href="javascript:openTermsAndConditions();">' + sTermsAndConditions + '</a>',
            padding : '0 0 5 0', //top right bottom left
            style : {
                color : '#005138'
            },
            width : textFieldWitdh
        });
 
        var step1Text = Ext.create('Ext.Panel', {
            layout : {
                type : 'vbox'
            },
            border : 0,
            width : TextContainerWidth,
            items : [ step1Description, accountFoundHint,  accountNotFoundHint, linkTermsAndConditions]
        });
        
        //button 'Create your digitalSTROM Account'
        var step1Button = Ext.create('Ext.button.Button', {
            text : sCreateButtonLabel,
            width : buttonWidth,
            handler : function() {
                var sURL = buildRegistrationURL();
                document.location.href=sURL;
            }          
        });

        var buttonPanel = Ext.create('Ext.Panel', {
            layout : {
                type : 'vbox'
            },
            border : 0,
            padding : '0 0 0 10', //top right bottom left
            items : [ step1Button]
        });

        var step1Panel = Ext.create('Ext.Panel', {
            id : 'id_step1Panel',
            layout : {
                type : 'hbox',
                align : 'bottom'
            },
            border : 0,
            padding : '10 0 10 0', //top right bottom left
            items : [ step1Text, buttonPanel],
            /**
             * Set appropriate visibility of the account existence hints 
             * @param accountFound boolean true if an account was found, false otherwise
             */
            setAccountFound : function(accountFound) {
                if (accountFound) {
                    accountNotFoundHint.hide();
                    accountFoundHint.show();
                } else {
                    accountFoundHint.hide();
                    accountNotFoundHint.show();
                }
            },
            hideAccountHint : function() {
                accountNotFoundHint.hide();
                accountFoundHint.hide();
            }
        });
        return step1Panel;
    },

    /**
     * Creates the panel for step 2: connect dSS with account of login panel.
     * 
     * @return {Object} The instantiated panel object.
     */
   makeStep2Panel : function() {
        var step2Description = Ext.create('Ext.form.Panel', {
            border : 0,
            html : '<span style="color:#005138;">' 
                + sStep2Title + '<br>' + sStep2Text
                + '</span>',
            padding : '0 0 5 0', //top right bottom left
            style : {
                color : '#005138'
            },
            width : textFieldWitdh
        });

        var step2Text = Ext.create('Ext.Panel', {
            layout : {
                type : 'vbox'
            },
            border : 0,
            width : TextContainerWidth,
            items : [ step2Description]
        });

        //button 'Connect digitalSTROM-Server with Account'
        var step2Button = Ext.create('Ext.button.Button', {
            text : sConnectButtonLabel,
            width : buttonWidth,
            handler : function() {
                RemoteConnectivity.LoginDialog.show();
                RemoteConnectivity.LoginDialog.center();
            }            
        });
        
        var buttonPanel = Ext.create('Ext.Panel', {
            layout : {
                type : 'vbox'
            },
            border : 0,
            padding : '0 0 0 10', //top right bottom left
            items : [ step2Button]
        });

        var step2Panel = Ext.create('Ext.Panel', {
            layout : {
                type : 'hbox',
                align : 'bottom'
            },
            border : 0,
            padding : '10 0 10 0', //top right bottom left
            items : [ step2Text, buttonPanel]
        });
        return step2Panel;
    },
    
    /**
     * Creates the panel for step 3: Hint of use of login panel.
     * 
     * @return {Object} The instantiated panel object.
     */
   makeStep3Panel : function() {
        var step3Description = Ext.create('Ext.form.Panel', {
            border : 0,
            html : '<span style="color:#005138;">' 
                + sStep3Title +'<br>' + sStep3Text
                + '</span>',
            padding : '0 0 5 0', //top right bottom left
            style : {
                color : '#005138'
            },
            width : textFieldWitdh
        });
        var step3Panel = Ext.create('Ext.Panel', {
            layout : {
                type : 'hbox'
            },
            border : 0,
            padding : '10 0 10 0', //top right bottom left
            items : [ step3Description]
        });
        return step3Panel;
    },

    /**
     * Creates the panel for info panel of connected tab.
     * 
     * @return {Object} The instantiated panel object.
     */
    createInfoPanel : function() {
        var oReturn = Ext.create('Ext.form.Panel', {
            border : 0,
            width : connectionPanelWidth,
            padding : '0 0 10 0', //top right bottom left
            html : '<div id="subPanel_Infotext" style="color:#005138;">' + sInfoTextStr + '</div>',
            style : {
                color : '#005138'
            }
        });
        return oReturn;
    },

    /**
     * Creates the panel for status of connected tab.
     * 
     * @return {Object} The instantiated panel object.
     */
    createStatusPanel : function() {
        //display field for user name
        var accountInfo = Ext.create('Ext.form.field.Display', {
            border : 0,
            fieldLabel : sUserNameLabel,
            width : textFieldWitdh,
            labelWidth : labelWidth
        });
        //display field for connection state
        var connectionState = Ext.create('Ext.form.field.Display', {
            border : 0,
            fieldLabel : sStatusLabel,
            width : textFieldWitdh,
            labelWidth : labelWidth
        });
        
        var statusPanel = Ext.create('Ext.Panel', {
            id : 'id_statusPanel',
            layout : {
                type : 'vbox'
            },
            border : 0,
            width : connectionPanelWidth,
            padding : '0 0 10 0', //top right bottom left
            items : [accountInfo, connectionState],
            updateStatus : function (userName, connection) {
                accountInfo.setValue(userName);
                connectionState.setValue(connection);
            }
        });
        return statusPanel;
    },
    
    /**
     * Creates the panel for opening service platform of connected tab.
     * 
     * @return {Object} The instantiated panel object.
     */
    createServicePanel : function() {
        var serviceText = Ext.create('Ext.form.field.Display', {
            border : 0,
            fieldLabel : _(''),
            width : textFieldWitdh,
            labelWidth : 0,
            value : sServiceText
        });
        
        //button 'Open Service Platform'
        var serviceButton = Ext.create('Ext.button.Button', {
            text : sServiceButtonLabel,
            width : buttonWidth,
            handler : function() {
                //Open service platform link in new tab
                openLinkInNewTab(servicePlatformURL);
            }
        });
        
        var servicePanel = Ext.create('Ext.Panel', {
            border : 0,
            width : connectionPanelWidth,
            padding : '0 0 10 0', //top right bottom left
            layout : {
                type : 'hbox',
                align : 'middle'
            },
            items : [ serviceText, serviceButton ]
        });
        return servicePanel;

    },
    
    /**
     * Creates the panel for managing connections of connected tab.
     * 
     * @return {Object} The instantiated panel object.
     */
    createConnectionManagmentPanel : function() {
        var disconnectDescription = Ext.create('Ext.form.Panel', {
            border : 0,
            width : textField2Witdh,
            html : '<div id="subPanel_Infotext" style="color:#005138;">' + '<p>' + sConnectionManagementSection1 + '</p><p>' + sConnectionManagementSection2 +'</p></div>',
            style : {
                color : '#005138'
            }
        });

        var linkTermsAndConditions = Ext.create('Ext.form.Panel', {
            border : 0,
            html : '<a href="javascript:openTermsAndConditions();">' + sTermsAndConditions + '</a>',
            padding : '0 0 0 10', //top right bottom left
            style : {
                color : '#005138'
            },
            width : textField2Witdh
        });

        var connectionManagementText = Ext.create('Ext.Panel', {
            layout : {
                type : 'vbox'
            },
            border : 0,
            width : textContainer2Width,
            items : [disconnectDescription, linkTermsAndConditions]
        });
        
        //button 'Permanently Disconnect
        var disconnectButton = Ext.create('Ext.button.Button', {
            text : sDisconnectButtonLabel,
            width : buttonWidth,
            handler : function() {
                //step 1 of disabling remote connectivity: show confirmation dialog
                RemoteConnectivity.confirmationDialogDisable.showDialog(sConfirmDisconnect, sYes);
            }
        });
        
        return Ext.create('Ext.form.FieldSet', {
            title : sManageConnectionTitle,
            width : connectionPanelWidth,
            padding : '0 0 10 0', //top right bottom left
            layout : {
                type : 'hbox',
                align : 'bottom'
            },
            items : [connectionManagementText, disconnectButton ]
        });
    },

    /**
     * Creates the panel for configure backup of connected tab.
     * 
     * @return {Object} The instantiated panel object.
     */
    createBackupServicePanel : function() {
        var backupServiceDescription = Ext.create('Ext.form.Panel', {
            border : 0,
            width : textField2Witdh,
            html : '<div id="subPanel_Infotext" style="color:#005138;">' + '<p>' + sBackupServiceSection1 + '</p><p>' + sBackupServiceSection2 +'</p></div>',
            style : {
                color : '#005138'
            }
        });

        var autoBackupCheckbox = Ext.create('Ext.form.field.Checkbox', {
            width : textField2Witdh,
            padding : '0 0 0 5', // top, right, bottom, left
            boxLabel : sActivateAutoBackup,
            style : {
                color : '#005138'
            }
        });
        autoBackupCheckbox.addListener('change',function(oThis,newValue,oldValue,eOpts) {
            if (true === newValue) {
                dss.ajaxSyncRequest('/cgi-bin/remotebackupwrapper.sh', {autoupload: 'on'});                
            } else {
                dss.ajaxSyncRequest('/cgi-bin/remotebackupwrapper.sh', {autoupload: 'off'});
            }
        });
        

        var backupServiceText = Ext.create('Ext.Panel', {
            layout : {
                type : 'vbox'
            },
            border : 0,
            width : textContainer2Width,
            items : [backupServiceDescription, autoBackupCheckbox]
        });
        
        var openBackupButton = Ext.create('Ext.button.Button', {
            text : sOpenBackupButtonLabel,
            width : buttonWidth,
            handler : function() {
                openLinkInNewTab(dataBackupURL);
            }
        });
        
        return Ext.create('Ext.form.FieldSet', {
            title : sBackupServiceTitle,
            id : 'id_backupServicePanel',
            width : connectionPanelWidth,
            padding : '0 0 10 0', //top right bottom left
            layout : {
                type : 'hbox',
                align : 'bottom'
            },
            items : [backupServiceText, openBackupButton ],
            updateStatus : function(newState) {
                autoBackupCheckbox.setValue(newState);
            },
            enableBackup : function() {
                autoBackupCheckbox.setValue(true);
                //dss.ajaxSyncRequest('/cgi-bin/remotebackupwrapper.sh', {autoupload: 'on'});
            },
            disableBackup : function() {
                autoBackupCheckbox.setValue(false);
                //dss.ajaxSyncRequest('/cgi-bin/remotebackupwrapper.sh', {autoupload: 'off'});   
            }
        });
    },
 
    /**
     * Creates the panel for temporary access.
     * 
     * @return {Object} The instantiated panel object.
     */
    createTempAccessPanel : function() {
        var tempAccessDescription = Ext.create('Ext.form.Panel', {
            border : 0,
            width : textField2Witdh,
            html : '<div id="subPanel_Infotext" style="color:#005138;">' + '<p>' + sTempServiceDescP1 + '</p>' +'<p>' + sTempServiceDescP2 + '</p>' +'<p>' + sTempServiceDescP3 + '</p></div>',
            style : {
                color : '#005138'
            }
        });

        var timePeriodStore = Ext.create('Ext.data.Store', {
            fields: ['id', 'name'],
            data: [{
                id: '2',
                name: _("2 ") + sUnit
            }, {
                id: '6',
                name: _("6 ") + sUnit
            }, {
                id: '12',
                name: _("12 ") + sUnit
            }]
        });

        var timePeriodBox = Ext.create('Ext.form.field.ComboBox', {
            store: timePeriodStore,
            editable: false,
            displayField: 'name',
            valueField: 'id',
            fieldLabel : sTimePeriodLabel,
            width : 300,
            labelWidth : labelWidth,
            padding : '0 0 0 5' // top, right, bottom, left
        });
        
        var tempAccessExists = Ext.create('Ext.form.Panel', {
            border : 0,
            width : textField2Witdh,
            html : '<div id="subPanel_Infotext" style="color:#005138;">' + '<p>' + sHintExists + '</p></div>',
            style : {
                color : '#005138'
            },
            hidden : true
        });
        
        var expirationTime = Ext.create('Ext.form.field.Display', {
            border : 0,
            fieldLabel : sExpirationTimeLabel,
            width : textFieldWitdh,
            labelWidth : labelWidth,
            padding : '0 0 0 5', // top, right, bottom, left
            hidden : true
        });

        var tempAccessText = Ext.create('Ext.Panel', {
            layout : {
                type : 'vbox'
            },
            border : 0,
            width : textFieldWitdh,
            items : [tempAccessDescription, timePeriodBox, tempAccessExists, expirationTime]
        });
        
        var sendMailButton = Ext.create('Ext.button.Button', {
            text : sSendMailButton,
            width : buttonWidth,
            handler : function() {
                RemoteConnectivity.activateTempConnection(timePeriodBox.getValue());
            }
        });

        var resendMailButton = Ext.create('Ext.button.Button', {
            text : sResendMailButton,
            width : buttonWidth,
            handler : function() {
                var expirationTime = getExpirationTime();
                var expirationDate = new Date(expirationTime);
                var currentDate = new Date();
                if (currentDate < expirationDate) {
                    resendMail(getTempLink(), formatExpirationTime(expirationTime));
                } else {
                    RemoteConnectivity.errorDialog.showWindow(sTempLinkExpired);
                    RemoteConnectivity.checkTempAccess();
                }
                
            },
            hidden : true
        });
        
        return Ext.create('Ext.form.FieldSet', {
            title : sTempServiceTtile,
            id : 'id_tempAccessPanel',
            width : connectionPanelWidth,
            padding : '0 0 10 0', //top right bottom left
            layout : {
                type : 'hbox',
                align : 'bottom'
            },
            items : [tempAccessText, sendMailButton, resendMailButton],
            setTempAccess : function(state, validity, expiring) {
                if ('active' === state) {
                    tempAccessExists.show();
                    expirationTime.setValue(formatExpirationTime(expiring));
                    expirationTime.show();
                    timePeriodBox.hide();
                    sendMailButton.hide();
                    resendMailButton.show();
                } else {
                    tempAccessExists.hide();
                    expirationTime.hide();
                    resendMailButton.hide();
                    sendMailButton.show();
                    timePeriodBox.setValue(validity);
                    timePeriodBox.show();
                }
                this.show();
            }
        });
    },
    
	/**
	 * The content generation function
	 * 
	 * @return {Object} The instantiated content object.
	 */
	getContent : function() {
		var me = this;
		
		//login tab
		var step1Panel = this.makeStep1Panel();
		var step2Panel = this.makeStep2Panel();
		var step3Panel = this.makeStep3Panel();
		this.loginTab = Ext.create('Ext.panel.Panel', {
			title : sLoginTab,
			id : 'login',
			style : {
				padding : '10px' //top
			},
			autoScroll : true,
			items : [{  xtype : 'fieldset',
			            id : 'intro',
			            title : _(''),
		                border : 0,
			            style: { padding:'10px', //top
			                    color: '#005138'
			            },
			            items : [ {
			            xtype : 'panel',
			                    border : 0,
			                    style: {color: '#005138'},
			                    width : 1000,
			                    html : dss.ajaxSyncRequest('locale/' + dss.staticDataModel.activeLanguage + '/intro.html',{})
			                }]
                    }, step1Panel, step2Panel, step3Panel]
		});
		
		//connected tab
		var infoPanel = this.createInfoPanel();
		var statusPanel = this.createStatusPanel();
		var servicePanel = this.createServicePanel();
		var connectionManagmentPanel = this.createConnectionManagmentPanel();
		var backupServicePanel = this.createBackupServicePanel();
		var tempAccessPanel = this.createTempAccessPanel();
		
        this.connectedTab = Ext.create('Ext.panel.Panel', {
            title : sConnectedTab,
            id : 'connected',
            style : {
                padding : '10px' //top
            },
            autoScroll : true,
            items : [infoPanel, statusPanel, servicePanel, tempAccessPanel, connectionManagmentPanel, backupServicePanel]
        });

        this.tabPanel = Ext.create('Ext.tab.Panel', {
			layout : 'fit',
			cls : 'generalTab',
			items : [ this.loginTab, this.connectedTab ]
		});
        
        return this.tabPanel;
	},

    /**
     * Verification weather a digitalstrom.com user account for the MAC address is existing.
     * Asynchronous call, result will be processed in waitAndReload() method
     */
    accountVerification : function() {
        this.waitAndReload('CheckUserByMAC');
        this.waitWindow.showWindow(sWaitAccountVerification, true);
        dss.raiseEvent('system-addon-remote-connectivity', {
            actions : 'CheckUserByMAC'
        });
    },

    /**
     * Disconnect from remote service.
     * Calls the cgi script on the dss which removes the tokens disable pagekite 
     * and call DeactivateRemoteConnectivity on cloud service
     */
    disconnectService: function() {
        RemoteConnectivity.waitAndReload('DeactivateRemoteConnectivity');
        RemoteConnectivity.waitWindow.showWindow(sWaitDisconnect, true);
        //disconnect from remote connectivity: send request to dSS
        dss.ajaxAssyncRequestWithoutAnswer('/cgi-bin/disablewebservice.sh', {});
    },

    /**
     * Connect to remote service.
     * Calls the cgi script on the dss which creates the tokens, activates pagekite
     * and call ActivateRemoteConnectivity on cloud service
     */
    activateRCService: function() {
        RemoteConnectivity.waitAndReload('ActivateRemoteConnectivity');
        //hide the login window and show a wait dialog instead
        RemoteConnectivity.confirmationDialogRCRestore.hide();
        //show wait window
        RemoteConnectivity.waitWindow.showWindow(sWaitConnecting, true);

        //enabling remote connectivity: send request to dSS
        dss.ajaxAssyncRequestWithoutAnswer('/cgi-bin/authservicewrapper.sh',{username:RemoteConnectivity.userAccount,password:RemoteConnectivity.userPwd,datasharing:2,restore:"no"});
    },

    /**
     * Restore remote service.
     * Calls the cgi script on the dss which restores the tokens and pagekite settings
     * afrom the cloud service
     */
    restoreRCService: function() {
        RemoteConnectivity.waitAndReload('ActivateRemoteConnectivity');
        //hide the login window and show a wait dialog instead
        RemoteConnectivity.confirmationDialogRCRestore.hide();
        //show wait window
        RemoteConnectivity.waitWindow.showWindow(sWaitRestoringConnection, true);

        //enabling remote connectivity: send request to dSS
        dss.ajaxAssyncRequestWithoutAnswer('/cgi-bin/authservicewrapper.sh',{username:RemoteConnectivity.userAccount,password:RemoteConnectivity.userPwd,datasharing:2,restore:"yes"});
    },
    
    /**
     * Call the cgi-script on dSS to activates a temporary pagekite link.
     * The generated link is only valid for timeperiod hours
     * Once the link is activated we will get an event with the link
     */
    activateTempConnection: function(timeperiod) {
        this.waitAndReload('ActivateTempLink');
        this.waitWindow.showWindow(sWaitTempLink, true);
        //send request to dSS
        dss.ajaxAssyncRequestWithoutAnswer('/cgi-bin/templinkwrapper.sh', {validity:timeperiod});
        this.tempLinkValidity = timeperiod;
    },
    
    /**
     * Check wether a temporary link already exists
     * Asynchronous call, result will be processed in waitAndReload() method
     */
    checkTempAccess: function() {
        this.waitAndReload('CheckExistingTempdSConfiguratorLink');
        this.waitWindow.showWindow(sWaitTempLinkVerification, true);        
        dss.raiseEvent('system-addon-remote-connectivity', {
            actions : 'CheckExistingTempdSConfiguratorLink'
        });
    },
    
    processTempAccessResult: function(linkExists, expirationTime) {
        if ('false' == linkExists) {
            RemoteConnectivity.connectedTab.items.getByKey('id_tempAccessPanel').setTempAccess('inactive', this.tempLinkValidity, '');
        } else {
            //temp link already exists
            RemoteConnectivity.connectedTab.items.getByKey('id_tempAccessPanel').setTempAccess('active', '', expirationTime);
            installAutoRefresh(expirationTime);
        }
    },
    
	/**
     * Event handler for back-end events. 
     */
	waitAndReload : function(action) {
	    var me = this;
	    var iSubscriptDemo = Math.floor(Math.random() * 100000000000);
	    dss.ajaxSyncRequest('/json/event/subscribe', {
	        subscriptionID : iSubscriptDemo,
	        name : 'system-addon-remote-connectivity.answer'
	    });
	    dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
	        subscriptionID : iSubscriptDemo,
	        timeout : 60 * 5000
	    }, function(answer) {   
	        var result = Ext.JSON.decode(answer,true);
	        if (result.ok==true) {
	            //first of all unsubscribe
	            dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
	                subscriptionID : iSubscriptDemo,
	                name : 'system-addon-remote-connectivity.answer'
	            });

	            if ((result.result!=null) && (result.result.events!=null) && (0<result.result.events.length)) {
	                for (var iIndex=0;iIndex<result.result.events.length;iIndex++) {
	                    if ('CheckUserByMAC' === result.result.events[iIndex].properties.actions) {
	                        checkUserResultHandler(result.result.events[iIndex].properties);
	                    }
                        if ('ActivateRemoteConnectivity' === result.result.events[iIndex].properties.actions) {
                            activateResultHandler(result.result.events[iIndex].properties);
                        }
	                    if ('DeactivateRemoteConnectivity' === result.result.events[iIndex].properties.actions) {
	                        deactivateResultHandler(result.result.events[iIndex].properties);
	                    }
	                    if ('ActivateTempLink' === result.result.events[iIndex].properties.actions) {
	                        activateTempLinkResultHandler(result.result.events[iIndex].properties);
	                    }
	                    if ('CheckExistingTempdSConfiguratorLink' === result.result.events[iIndex].properties.actions) {
	                        checkTempLinkHandler(result.result.events[iIndex].properties);
	                    }
	                    if ('checkExistingConnection' === result.result.events[iIndex].properties.actions) {
	                        checkExistingConnectionHandler(result.result.events[iIndex].properties);
	                    }
	                }       
	            } else {
	                //timeout
	                var sAction;
	                if ('CheckUserByMAC' === action) {
	                    sAction = '<div>Action : check of user account</div>';
	                }
	                if ('ActivateRemoteConnectivity' === action) {
	                    sAction = '<div>Action : Connect</div>';
                    }
	                if ('DeactivateRemoteConnectivity' === action) {
	                    sAction = '<div>Action : Disconnect</div>';
                    }
	                if ('ActivateTempLink' === action) {
	                    sAction = '<div>Action : Activate temporary access</div>';
	                }
	                if ('CheckExistingTempdSConfiguratorLink' === action) {
	                    sAction = '<div>Action : Check existence of temporary link</div>';
	                }
	                if ('checkExistingConnection' === action) {
	                    sAction = '<div>Action : Check existence of remote connectivity configuration</div>';
	                }

                    me.waitWindow.closeWindow();
	                me.errorDialog.showWindow(sTimeout + sAction);
	            }
	        } else {
	            me.waitWindow.closeWindow();
	            showErrorMsg(sConnectErrorIntro, result.code, result.servicecode);
	        }
	    });
	},
	
	/**
	 * Check the current state of service
	 * @returns {Boolean} true if service is activated, false otherwise
	 */
	checkServiceState : function(){
        this.remoteConnectivityEnabled = false;
        var resultObj;
        var sResponseRemoteAccess = dss.ajaxSyncRequest('/cgi-bin/remoteaccessinfo.sh', {});
        try {
            resultObj = Ext.JSON.decode(sResponseRemoteAccess);
            if (resultObj.success) {
                if (null !== resultObj.data.remoteaccess) {
                    this.remoteConnectivityEnabled = resultObj.data.remoteaccess;
                }
            }
        }
        catch(e) {
            this.errorDialog.showWindow(sExceptionCheckServiceState);
        }
        return this.remoteConnectivityEnabled;
    },

    /**
     * Load the information whether auto backup is enabled from the dSS.
     */
    checkBackupStatus : function(){
        this.autoBackupActivated = false;
        var resultObj;
        var sResponseBackup = dss.ajaxSyncRequest('/cgi-bin/remotebackupwrapper.sh', {autoupload : 'status'}); 
        try {
            resultObj = Ext.JSON.decode(sResponseBackup);
            if (resultObj.success) {
                if (null !== resultObj.data.autouploadbackups) {
                    this.autoBackupActivated = resultObj.data.autouploadbackups;
                }
            } 
        } catch(e) {
            this.errorDialog.showWindow(sExceptionCheckBackupState);
        }
        return this.autoBackupActivated;
    }

});

Ext.onReady(function() {
	dss.buildUpLang(['locale/{languageSuffix}/remote-connectivity.po']);
	initStrings();
	initURLs();
	
	//remove relay link from property tree as this is not needed.
	dss.raiseEvent('system-addon-remote-connectivity', {
	    actions : 'cleanUpPropertyTree',
	});

	//create the Main window with 2 tabs
	RemoteConnectivity = Ext.create('DSS.addon.RemoteConnectivity');
    //create the different dialogs we need (see dialogs.js)
    RemoteConnectivity.LoginDialog = createLoginDialog();
    RemoteConnectivity.waitWindow = createWaitWindow();
    RemoteConnectivity.confirmationDialogDisable = createConfirmationDialog(RemoteConnectivity.disconnectService, undefined);
    RemoteConnectivity.confirmationDialogRCRestore = createConfirmationDialog(RemoteConnectivity.restoreRCService, RemoteConnectivity.activateRCService);
    RemoteConnectivity.errorDialog = createErrorDialog();
    RemoteConnectivity.waitWindow.showWindow(sInitialising, false);
    if (checkdSSVersion(minReqVersion, getPropertyString('/system/version/distroVersion'), getPropertyString('/system/version/releasetype'))) {
    //check service state
    if (true === RemoteConnectivity.checkServiceState()) {
        var backupState = RemoteConnectivity.checkBackupStatus();
        RemoteConnectivity.connectedTab.items.getByKey('id_backupServicePanel').updateStatus(backupState);
        RemoteConnectivity.userAccount = getPropertyString('/scripts/system-addon-remote-connectivity/username');
        RemoteConnectivity.connectedTab.items.getByKey('id_statusPanel').updateStatus(RemoteConnectivity.userAccount, sConnected);
        RemoteConnectivity.connectedTab.tab.setDisabled(false);
        RemoteConnectivity.tabPanel.setActiveTab(RemoteConnectivity.connectedTab);
        RemoteConnectivity.loginTab.tab.setDisabled(true);
        RemoteConnectivity.waitWindow.closeWindow();
        RemoteConnectivity.tempLinkValidity = getParameterByName('timeperiod');
        if (RemoteConnectivity.tempLinkValidity == '') {
            RemoteConnectivity.tempLinkValidity = defaultValidity;
        }
        RemoteConnectivity.connectedTab.items.getByKey('id_tempAccessPanel').setTempAccess('inactive', RemoteConnectivity.tempLinkValidity, '');
        RemoteConnectivity.checkTempAccess();
    } else {
        RemoteConnectivity.tempLinkValidity = defaultValidity;
        RemoteConnectivity.connectedTab.tab.setDisabled(true);
        //verify existence of account
        RemoteConnectivity.accountVerification();
    }
    RemoteConnectivity.initPage();
	} else {
	    RemoteConnectivity.errorDialog.showWindow(sWronVersion, true);
	}
    Ext.EventManager.on(window, 'beforeunload', function() {
        cleanUp();
    });
});
